[% WRAPPER page.tt %]
  <h2>ISO Images</h2>

  [% IF isos && isos.size > 0 %]
    <table id="isos" class="data">
      <thead>
        <tr>
          <th>Filename</th>
          <th>Size</th>
          <th>MD5 Sum</th>
        </tr>
      </thead>
      <tbody>
        [% FOREACH isos %]
          <tr class="[% IF loop.index() % 2 == 0 %]odd[%ELSE%]even[%END%]">
            <th class="filename"><a href="isos/[% filename | html %]">[% label | html %]</a></th>
            <td class="size">[% size %]</td>
            <td class="md5sum big">[% md5sum %]</td>
          </tr>
        [% END %]
      </tbody>
    </table>
  [% ELSE %]
    <p>There are no ISO images defined</p>
  [% END %]
[% END %]
