/*
body {
  margin: 0em;
  padding: 0px;
  color: rgb(0,0,0);
  font-family: Verdana, Arial, Helvetica, sans-serif;
  font-size: 76%;
}

p, ul, ol, dl {
  padding: 0px;
  margin: 0px;
}

ol,ul {
  margin-left: 3em;
}

ol,ul,dl,p {
  margin-top: 1em;
  margin-bottom: 1em;
}

p:first-line {
  margin-right: 1em;
}

div.body p:first-letter {
  font-size: 1.2em;
  font-weight: bold;
}

h1,h2,h3,h4,h5,h6 {
  font-weight: bold;
  margin: 0px;
  padding: 0px;
  margin-top: 0.5em;
}

div.footer {
  margin-top: 1em;
}

h1 {
  font-size: 2em;
}
h2 {
  font-size: 1.6em;
}
h3 {
  font-size: 1.4em;
}
h4 {
  font-size: 1.2em;
}
h5 {
  font-size: 1em;
}
h6 {
  font-size: 0.8em;
}

dl dt {
  margin-left: 1em;
  font-weight: bold;
  font-size: larger;
  border-top: 1px dotted white;
}


dl dd {
  background: rgb(230,230,230);
  margin-right: 2em;
  margin-bottom: 0.5em;
}
*/